#ifndef FGGL_DATA_PROCEDURAL_HPP
#define FGGL_DATA_PROCEDURAL_HPP

#include "model.hpp"

namespace fggl::data {

	constexpr math::mat4 OFFSET_NONE(1.0f);

	// basic shapes 'building blocks'
	Mesh make_triangle();
	Mesh make_quad_xy();
	Mesh make_quad_xz();

	// platonic solids
	void make_tetrahedron(Mesh& mesh, const math::mat4& offset = OFFSET_NONE);
	Mesh make_cube(Mesh &mesh, const math::mat4 &offset = OFFSET_NONE);
	void make_octahedron(Mesh& mesh, const math::mat4& offset = OFFSET_NONE);
	void make_icosahedron(Mesh& mesh, const math::mat4& offset = OFFSET_NONE);
	void make_dodecahedron(Mesh& mesh, const math::mat4& offset = OFFSET_NONE);

	// useful shapes
	void make_sphere_uv(Mesh &mesh, const math::mat4& offset = OFFSET_NONE);
	void make_sphere_iso(Mesh &mesh, const math::mat4& offset = OFFSET_NONE);

	// level block-out shapes
	Mesh make_slope(Mesh &mesh, const math::mat4 &offset = OFFSET_NONE);
	Mesh make_ditch(Mesh &mesh, const math::mat4 &offset = OFFSET_NONE);
	Mesh make_point(Mesh &mesh, const math::mat4 &offset = OFFSET_NONE);

	// other useful types people expect
	void make_capsule(Mesh& mesh);
	void make_sphere(Mesh &mesh, const math::mat4& offset = OFFSET_NONE, int stacks = 16, int slices = 16);
}

#endif